/*
 *****************************************************
 *
 *  SaVi by Robert Thurman (thurman@geom.umn.edu) and
 *          Patrick Worfolk (worfolk@alum.mit.edu).
 *
 *  Copyright (c) 1997 by The Geometry Center.
 *  This file is part of SaVi.  SaVi is free software;
 *  you can redistribute it and/or modify it only under
 *  the terms given in the file COPYRIGHT which you should
 *  have received along with this file.  SaVi may be
 *  obtained by emailing savi@geom.umn.edu.
 *
 *****************************************************
 *
 * earth.c
 *
 * Routines for creating fancy earth model
 *
 * $Id: earth.c,v 1.1.1.1 2001/11/11 19:59:55 lloydwood Exp $
 */
#include <stdio.h>
#include <math.h>

#include "gv_utils.h"
#include "utils.h"
#include "constants.h"
#include "globals.h"
#include "savi.h"

static int earth_on_flag = FALSE;
static int earth_geom_exists = FALSE;
static int texture_flag = FALSE;
static int use_fancy_earth = FALSE;

/*
 * earth_on_cmd
 *
 * Displays the model of the earth
 */
char *
earth_on_cmd(int argc, char *argv[])
{
  char *status = EMPTY_str;

  if (TRUE == earth_on_flag)
    return status;
  earth_on_flag = TRUE;

  if (TRUE == use_fancy_earth) {
    /* make a fancy earth */
    use_fancy_earth = FALSE;
    status = fancy_on_cmd(argc, argv);
  } else {
    if (TRUE == geomview_module) {
      /* create a simple blue sphere for the earth */
      gv_begin();
      gv_create_geom("Central_Body", "central_t", "unit_sphere_h");
      gv_end();
    }
  }

  earth_geom_exists = TRUE;

  return status;
}

/*
 * earth_off_cmd
 *
 * Uses a sphere to model the central body
 */
char *
earth_off_cmd(int argc, char *argv[])
{
  if (FALSE == earth_on_flag)
    return EMPTY_str;

  earth_delete_geom();
  earth_on_flag = FALSE;

  return EMPTY_str;
}

/*
 * earth_on_cmd
 *
 * Creates a fancy model of the earth by loading vector
 * or texture-map file.
 */
char *
fancy_on_cmd(int argc, char *argv[])
{
  int ok = TRUE;
  char *status = EMPTY_str;

  if (TRUE == use_fancy_earth)
    return status;

  use_fancy_earth = TRUE;

  if ((TRUE == geomview_module) && (TRUE == earth_on_flag)) {
    gv_begin();
    if (TRUE == texture_flag) {
      ok = gv_sendfile("oogl/earth_h.oogl");
    } else {
      ok = gv_sendfile("oogl/earth_vect_h.oogl");
    }
    if (TRUE == ok) {
      gv_create_geom("Central_Body", "central_t", "earth_h");
    }
    gv_end();
  }

  return status;
}


/*
 * fancy_off_cmd
 *
 * Uses a simple sphere to model the central body
 */
char *
fancy_off_cmd(int argc, char *argv[])
{
  char *status = EMPTY_str;

  if (FALSE == use_fancy_earth)
    return status;

  use_fancy_earth = FALSE;

  if (TRUE == earth_on_flag) {
    status = earth_off_cmd(argc, argv);
    status = earth_on_cmd(argc, argv);
  }

  return status;
}

/*
 * texture_on
 *
 * Use texture mapping for fancy earth model
 */
char *
texture_on_cmd(int argc, char *argv[])
{
  char *status = EMPTY_str;

  texture_flag = TRUE;

  if ((TRUE == use_fancy_earth) && (TRUE == earth_on_flag)) {
    status = earth_off_cmd(argc, argv);
    status = earth_on_cmd(argc, argv);
  }

  return status;
}

/*
 * texture_off_cmd
 *
 * Use vect model for fancy earth model
 */
char *
texture_off_cmd(int argc, char *argv[])
{
  char *status = EMPTY_str;

  texture_flag = FALSE;

  if ((TRUE == use_fancy_earth) && (TRUE == earth_on_flag)) {
    status = earth_off_cmd(argc, argv);
    status = earth_on_cmd(argc, argv);
  }

  return status;
}


/*
 * earth_place
 *
 * rotates the earth to correct position for time t
 */
void
earth_place(double t, const CentralBody * pcb)
{
  double m[4][4], theta, ct, st;

  if (FALSE == earth_on_flag)
    return;

  identity(m);

  theta = t * pcb->rotation_rate;

  ct = cos(theta);
  st = sin(theta);

  m[0][0] = ct;
  m[0][1] = st;
  m[1][0] = -st;
  m[1][1] = ct;
  if (TRUE == geomview_module) {
    gv_transform("central_t", m);
  }
}


/*
 * earth_gv_delete
 */
void
earth_delete_geom()
{
  if (FALSE == earth_geom_exists)
    return;

  if (TRUE == geomview_module) {
    gv_begin();
    gv_delete_geom("Central_Body");
    gv_end();
  }
  earth_geom_exists = FALSE;

}
